#!/bin/gawk -f
#
# Component: etc
#
# Description:
#   Evaluates commands in cronfigSchedule.cfg file(s) and prints values assigned to variables in a form that can be included to shell scripts
#
# Last Updated     : $Author$
# Update Date Time : $Date$
# RCS File Name    : $RCSfile$
# RCS Revision     : $Revision$
# Locked by        : $Locker$
# Symbolic name    : $Name$
# State            : $State$
#


#
# Bring time to canonical form:
# Hour: 0-23
# Minute: 0-59
#
function canonize(time) {
	split(time, tm, ":");

	tm[1] = tm[1] + 0;
	tm[2] = tm[2] + 0;

	isNegativeHours = 0;
	isNegativeMinutes = 0;
	if (tm[1] < 0) {
		isNegativeHours = 1;
	}
	if (tm[2] < 0) {
		isNegativeMinutes = 1;
	}

	mhours = int(tm[2] / 60);
	minutes = tm[2] % 60;

	sumhours = tm[1] + mhours
	hours = sumhours % 24;

	if (minutes < 0) {
		minutes = 60 + minutes;
		hours = hours - 1;
	}

	if (hours < 0) {
		hours = 24 + hours;
	}

	return hours ":" minutes;
}


#
# Converts "HH:MM" to "MM HH"
#
#
function toCronTime(time) {
	split(time, tm, ":");
	
	tm[1] = tm[1] + 0; tm[2] = tm[2] + 0;

	return tm[2] " " tm[1];
}
	

#
# Calculate the sum of two times. The result would be a 24 hours based time
#
function addTimes(time1,time2) {
	split(time1, tm1, ":");
	split(time2, tm2, ":");
	
	tm1[1] = tm1[1] + 0; tm1[2] = tm1[2] + 0;
	tm2[1] = tm2[1] + 0; tm2[2] = tm2[2] + 0;
	
	minutes = tm1[2] + tm2[2];
	hours = tm1[1] + tm2[1];
	
	return canonize(hours ":" minutes)
}

function substractTimes(time1,time2) {
	split(time1, tm1, ":");
	split(time2, tm2, ":");
	
	tm1[1] = tm1[1] + 0; tm1[2] = tm1[2] + 0;
	tm2[1] = tm2[1] + 0; tm2[2] = tm2[2] + 0;
	
	minutes = tm1[2] - tm2[2];
	hours = tm1[1] - tm2[1];

	return canonize(hours ":" minutes)
}

BEGIN {
	# For iterating through assigned times in order
	numTimes = 0;
}

# Skip comment lines
/^#/ {
	next;
}

# Skip empty lines
($1 == "") {
	next;
}

# Defining time for periodical activities
($2 == ":") {
	if ($3 == "")
		next;

	if (periodArray[$1] != "" || timeArray[$1] != "") {
		# There is already a value assigned to this variable - skip
		next;
	}

	result[numTimes++] = $1;

	if ( $4 == "" )
		periodArray[$1] = $3 " *";
	else
		periodArray[$1] = $3 " " $4;
	
	vExtra = "";
	for ( i = 5; i < 8; i++ ) {
		if ( $i == "" )
			vExtra = vExtra " *";
		else
			vExtra = vExtra " " $i;
	}
	extraArray[ $1 ] = vExtra;
}


# Skip lines with no assignment operator
($2 != "=") {
	next;
}


# operators
( $4 == "+" || $4 == "-" ) {
	if ( NF < 5 ) {
		# Not enough argument for operator
		next
	}

	if ( periodArray[ $1 ] != "" || timeArray[ $1 ] != "" ) {
		# There is already a value assigned to this variable - skip
		next;
	}

	t1 = timeArray[ $3 ];
	if ( t1 == "" )
	  t1 = $3;

	t2 = timeArray[ $5 ];
	if ( t2 == "" )
		t2 = $5;	
	
	if ( $4 == "+" ) {
		time = addTimes( t1, t2 );
	}
	else {
		time = substractTimes( t1, t2 );
	}

	result[ numTimes++ ] = $1;
	timeArray[ $1 ] = time;

	vExtra = "";
	for ( i = 6; i < 9; i++ ) {
		if ( $i == "" )
			vExtra = vExtra " *";
		else
			vExtra = vExtra " " $i;
	}
	extraArray[ $1 ] = vExtra;
	next;
}

# Single value assignment (tm = t1)
{
	
	if (periodArray[$1] != "" || timeArray[$1] != "") {
		# There is already a value assigned to this variable - skip
		next;
	}

	timeArray[$1] = $3;
	result[numTimes++] = $1;

	vExtra = "";
	for ( i = 4; i < 7; i++ ) {
		if ( $i == "" )
			vExtra = vExtra " *";
		else
			vExtra = vExtra " " $i;
	}
	extraArray[ $1 ] = vExtra;
	next;
}


END {
	for (i = 0; i < numTimes; i++) {
		tName = result[i];
		if (timeArray[tName] == "") {
			print tName "=\"" periodArray[tName] "\"";
			print tName "Full=\"" periodArray[tName] extraArray[tName] "\"";
		}
		else {
			print tName "=\"" toCronTime(timeArray[tName]) "\"";
			print tName "Full=\"" toCronTime(timeArray[tName]) extraArray[tName] "\"";
		}
	}
}
